﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using Infragistics.Controls.Charts;
using System.Windows.Ink;
using Infragistics.Windows;
using System.Windows.Controls.Primitives;

namespace DataChartScatterLineSeries
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        private ScatterTooltipManager<TestDataItem> _manager;

        private void theChart_Loaded(object sender, RoutedEventArgs e)
        {
            _manager = new ScatterTooltipManager<TestDataItem>(
                theChart,
                theChart.Series[0] as ScatterBase,
                (i) => i.XValue,
                (i) => i.YValue,
                thePopup);
        }
    }

    public class ScatterTooltipManager<T>
    {
        private ScatterBase _scatterSeries = null;
        private Func<T, double> _xMapping = null;
        private Func<T, double> _yMapping = null;
        private Popup _tooltipContainer = null;

        public ScatterTooltipManager(
            XamDataChart chart,
            ScatterBase scatterSeries,
            Func<T, double> xMapping,
            Func<T, double> yMapping,
            Popup tooltipContainer)
        {
            chart.SeriesMouseMove += new DataChartMouseEventHandler(chart_SeriesMouseMove);
            chart.SeriesMouseLeave += new DataChartMouseEventHandler(chart_SeriesMouseLeave);
            _scatterSeries = scatterSeries;
            _xMapping = xMapping;
            _yMapping = yMapping;
            _tooltipContainer = tooltipContainer;
        }

        void chart_SeriesMouseLeave(object sender, ChartMouseEventArgs e)
        {
            if (e.Series == _scatterSeries)
            {
                _tooltipContainer.IsOpen = false;
            }
        }

        void chart_SeriesMouseMove(object sender, ChartMouseEventArgs e)
        {
            if (e.Series != _scatterSeries)
            {
                return;
            }

            var item = FindClosestItem(e);
            var pos = e.GetPosition(_scatterSeries);

            _tooltipContainer.Placement = PlacementMode.Relative;
            _tooltipContainer.PlacementTarget = _scatterSeries;
            _tooltipContainer.HorizontalOffset = pos.X + 10;
            _tooltipContainer.VerticalOffset = pos.Y + 10;
            _tooltipContainer.DataContext = item;
            _tooltipContainer.IsOpen = true;
        }

        private TestDataItem FindClosestItem(ChartMouseEventArgs e)
        {
            var pos = e.GetPosition(_scatterSeries);
            var viewport = new Rect(0, 0, _scatterSeries.ActualWidth, _scatterSeries.ActualHeight);
            double x = _scatterSeries.XAxis.GetUnscaledValue(
                pos.X, new ScalerParams(
                e.Chart.WindowRect,
                viewport,false));
            double y = _scatterSeries.YAxis.GetUnscaledValue(
                pos.Y,
                new ScalerParams(
                e.Chart.WindowRect,
                viewport, false));


            TestDataItem minItem = new TestDataItem() { XValue = Math.Round(x, 2), YValue = Math.Round(y, 2) };
       

            return minItem;
        }

        private double GetDistance(double itemX, double itemY, double x, double y)
        {
            return Math.Sqrt(Math.Pow(itemX - x, 2) + Math.Pow(itemY - y, 2));
        }
    }

    public class TestDataItem
    {
        public double XValue { get; set; }
        public double YValue { get; set; }
    }

    public class TestData
        : ObservableCollection<TestDataItem>
    {
        public TestData()
        {
            Add(new TestDataItem()
            {
                XValue = 0,
                YValue = 0
            });
            Add(new TestDataItem()
            {
                XValue = 1,
                YValue = 1
            });
            Add(new TestDataItem()
            {
                XValue = 2,
                YValue = 2
            });
            Add(new TestDataItem()
            {
                XValue = 3,
                YValue = 3
            });
            Add(new TestDataItem()
            {
                XValue = 4,
                YValue = 4
            });
            Add(new TestDataItem()
            {
                XValue = 5,
                YValue = 5
            });
        }
    }   
}
